@@ -50,7 +50,6 @@
 
             fail_if/1,				% :Goal
 
-            not_exists/1,			% :Goal
             sk_not/1,				% :Goal
             gc_tables/1,                        % -Remaining
 
@@ -58,6 +57,8 @@
             walltime/1,				% -Seconds
             timed_call/2,                       % :Goal, :Options
 
+            (thread_shared)/1,                  % :Spec
+
             debug_ctl/2,                        % +Option, +Value
 
             fmt_write/2,                        % +Fmt, +Term
@@ -66,6 +67,8 @@
             path_sysop/2,                       % +Op, ?Value
             path_sysop/3,                       % +Op, ?Value1, ?Value2
 
+            abort/1,				% +Message
+
             op(1050,  fy, import),
             op(1050,  fx, export),
             op(1040, xfx, from),
@@ -74,7 +77,8 @@
             op(1100,  fx, mode),                % ignored
             op(1045, xfx, as),
             op(900,   fy, tnot),
-            op(900,   fy, not)                  % defined as op in XSB
+            op(900,   fy, not),                 % defined as op in XSB
+            op(1100,  fx, thread_shared)
           ]).
 :- use_module(library(error)).
 :- use_module(library(debug)).
@@ -100,8 +104,9 @@ system](http://xsb.sourceforge.net/)
     load_dync(:),
     load_dync(:, +),
 
+    thread_shared(:),
+
     fail_if(0),                         % Meta predicates
-    not_exists(0),
     sk_not(0).
 
 
@@ -470,15 +475,11 @@ fail_if(P) :-
 		 *      TABLING BUILT-INS	*
 		 *******************************/
 
-%!  not_exists(:P).
-%!  sk_not(:P).
+%!  sk_not(:P) is semidet.
 %
-%   XSB tabled negation. According to the XSB manual, sk_not/1 is an old
-%   name for not_exists/1. The predicates   tnot/1  and not_exists/1 are
-%   not precisely the same. We ignore that for now.
-
-not_exists(P) :-
-    tnot(P).
+%   Sound negation with non-ground P.  Equivalent to not_exists/1.
+%
+%   @deprecated New code should use not_exists/1.
 
 sk_not(P) :-
     not_exists(P).
@@ -536,6 +537,15 @@ debug_ctl(Option, Value) :-
     debug(xsb(compat), 'XSB: not implemented: ~p',
           [ debug_ctl(Option, Value) ]).
 
+%!  thread_shared(+Spec)
+%
+%   Declare a dynamic predicate  as  shared.   This  is  the default for
+%   SWI-Prolog.
+
+thread_shared(Spec) :-
+    dynamic(Spec).
+
+
 %!  fmt_write(+Fmt, +Term) is det.
 %!  fmt_write(+Stream, +Fmt, +Term) is det.
 %
@@ -672,6 +682,13 @@ path_sysop(dirname, Path, Dir) :-
 path_sysop(expand, Name, Path) :-
     absolute_file_name(Name, Path).
 
+%!  abort(+Message:atomic)
+%
+%   Abort with a message
+
+abort(Message) :-
+    print_message(error, aborted(Message)),
+    abort.
 
 		 /*******************************
 		 *           MESSAGES		*
