@@ -99,6 +99,7 @@ source file is passed into _Where.
                                                 test,development]))),
                        source(boolean),
                        trace_reference(any),
+                       trace_condition(callable),
                        on_trace(callable),
                        infer_meta_predicates(oneof([false,true,all])),
                        evaluate(boolean),
@@ -115,6 +116,7 @@ source file is passed into _Where.
                 infer_meta_predicates:oneof([false,true,all])=true,
                 clauses:list,               % Walk only these clauses
                 trace_reference:any=(-),
+                trace_condition:callable,   % Call-back condition
                 on_trace:callable,          % Call-back on trace hits
                                             % private stuff
                 clause,                     % Processed clause
@@ -176,6 +178,23 @@ source file is passed into _Where.
 %     represented as Module:Callable (i.e., they are always
 %     qualified).  See also subsumes_term/2.
 %
+%     * trace_condition(:Cond)
+%     Additional filter condition applied after `trace_reference`.
+%     Called as call(Cond, Callee, Context), where `Context` is a
+%     dict containing the following keys:
+%
+%       - Context:caller
+%         Qualified term representing the caller or the atom
+%         '<initialization>'.
+%       - Context:module
+%         Module being processed
+%       - Context:clause
+%         If we are processing a normal clause, the clause reference
+%         to this clause.
+%       - Context:initialization
+%         If we are processing an initialization/1 directive, a term
+%         `File:Line` representing the location of the declaration.
+%
 %     * on_trace(:OnTrace)
 %     If a reference to =trace_reference= is found, call
 %     call(OnTrace, Callee, Caller, Location), where Location is one
@@ -235,7 +254,7 @@ prolog_walk_code(Iteration, Options) :-
     ).
 
 is_meta(on_trace).
-
+is_meta(trace_condition).
 
 %!  walk_clauses(+Clauses, +OTerm) is det.
 %
@@ -291,7 +310,8 @@ init_goal_in_scope(Goal, SourceLocation, OTerm) :-
     ;   walk_option_module_class(OTerm, Classes),
         source_file_property(File, module(MF))
     ->  module_property(MF, class(Class)),
-        memberchk(Class, Classes)
+        memberchk(Class, Classes),
+        walk_option_module(OTerm, MF)
     ;   true
     ).
 
@@ -309,6 +329,7 @@ walk_from_initialization(_, _).
 
 find_walk_from_module(M, OTerm) :-
     debug(autoload, 'Analysing module ~q', [M]),
+    walk_option_module(OTerm, M),
     forall(predicate_in_module(M, PI),
            walk_called_by_pred(M:PI, OTerm)).
 
@@ -512,6 +533,7 @@ walk_called(Goal, Module, TermPos, OTerm) :-
     ;   predicate_property(Module:Goal, imported_from(M2)),
         subsumes_term(To, M2:Goal)
     ),
+    trace_condition(M2:Goal, TermPos, OTerm),
     print_reference(M2:Goal, TermPos, trace, OTerm),
     fail.                                   % Continue search
 walk_called(Goal, Module, _, OTerm) :-
@@ -566,6 +588,29 @@ walk_called(Goal, Module, TermPos, OTerm) :-
 walk_called(Goal, _Module, TermPos, OTerm) :-
     not_callable(Goal, TermPos, OTerm).
 
+%!  trace_condition(:Callee, +TermPos, +OTerm) is semidet.
+%
+%   Call call(Condition, Callee, Dict)
+
+trace_condition(Callee, TermPos, OTerm) :-
+    walk_option_trace_condition(OTerm, Cond), nonvar(Cond),
+    !,
+    cond_location_context(OTerm, TermPos, Context0),
+    walk_option_caller(OTerm, Caller),
+    walk_option_module(OTerm, Module),
+    put_dict(#{caller:Caller, module:Module}, Context0, Context),
+    call(Cond, Callee, Context).
+trace_condition(_, _, _).
+
+cond_location_context(OTerm, _TermPos, Context) :-
+    walk_option_clause(OTerm, Clause), nonvar(Clause),
+    !,
+    Context = #{clause:Clause}.
+cond_location_context(OTerm, _TermPos, Context) :-
+    walk_option_initialization(OTerm, Init), nonvar(Init),
+    !,
+    Context = #{initialization:Init}.
+
 %!  undecided(+Variable, +TermPos, +OTerm)
 
 undecided(Var, TermPos, OTerm) :-
@@ -892,6 +937,8 @@ walk_dcg_body(G, M, TermPos, OTerm) :-
 :- meta_predicate
     subterm_pos(+, +, 2, +, -),
     sublist_pos(+, +, +, +, 2, -).
+:- public
+    subterm_pos/5.                      % used in library(check).
 
 subterm_pos(_, _, _, Pos, _) :-
     var(Pos), !, fail.
