@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2013-2019, VU University Amsterdam
+    Copyright (c)  2013-2020, VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
 
@@ -37,19 +37,18 @@
           [ safe_goal/1,                % :Goal
             safe_call/1                 % :Goal
           ]).
-:- autoload(library(apply),[maplist/2]).
-:- autoload(library(assoc),[empty_assoc/1,get_assoc/3,put_assoc/4]).
-:- autoload(library(debug),[debug/3,debugging/1]).
-:- autoload(library(error),
-	    [ must_be/2,
-	      instantiation_error/1,
-	      type_error/2,
-	      permission_error/3
-	    ]).
-:- autoload(library(lists),[append/3]).
-:- autoload(library(prolog_format),[format_types/2]).
 :- use_module(library(apply_macros),[expand_phrase/2]).
-
+:- use_module(library(apply),[maplist/2]).
+:- use_module(library(assoc),[empty_assoc/1,get_assoc/3,put_assoc/4]).
+:- use_module(library(debug),[debug/3,debugging/1]).
+:- use_module(library(error),
+              [ must_be/2,
+                instantiation_error/1,
+                type_error/2,
+                permission_error/3
+              ]).
+:- use_module(library(lists),[append/3]).
+:- use_module(library(prolog_format),[format_types/2]).
 
 :- multifile
     safe_primitive/1,               % Goal
