@@ -205,7 +205,6 @@ struct PL_global_data
   { size_t	highest;		/* Highest atom index */
     atom_array	array;
     AtomTable	table;			/* hash-table */
-    Atom	builtin_array;		/* Builtin atoms */
     int		lookups;		/* # atom lookups */
     int		cmps;			/* # string compares for lookup */
     int		initialised;		/* atoms have been initialised */
@@ -387,6 +386,8 @@ struct PL_global_data
     PL_thread_info_t   *free;		/* Free threads */
     int			highest_allocated; /* Highest with info struct */
     int			thread_max;	/* Size of threads array */
+    int			highest_id;	/* Highest Id of life thread  */
+    int			peak_id;	/* Highest Id of any thread  */
     PL_thread_info_t  **threads;	/* Pointers to thread-info */
     struct
     { pthread_mutex_t	mutex;
