@@ -112,7 +112,8 @@ PL_wait_for_console_input(void *handle)
     } else if ( rc == WAIT_OBJECT_0 )
     { return TRUE;
     } else
-    { Sdprintf("MsgWaitForMultipleObjects(): 0x%x\n", rc);
+    { DEBUG(MSG_WIN_API,
+	    Sdprintf("MsgWaitForMultipleObjects(): 0x%x\n", rc));
     }
   }
 }
@@ -332,7 +333,7 @@ findExecutable(const char *module, char *exe, size_t exelen)
   if ( module )
   { if ( !(hmod = GetModuleHandle(module)) )
     { hmod = GetModuleHandle("libswipl.dll");
-      DEBUG(0,
+      DEBUG(MSG_WIN_API,
 	    Sdprintf("Warning: could not find module from \"%s\"\n"
 		     "Warning: Trying %s to find home\n",
 		     module,
@@ -668,6 +669,11 @@ load_library_search_flags(void)
 	 (f_RemoveDllDirectory = (void*)GetProcAddress(kernel, "RemoveDllDirectory")) )
     { flags = ( LOAD_LIBRARY_SEARCH_DLL_LOAD_DIR|
 		LOAD_LIBRARY_SEARCH_DEFAULT_DIRS );
+      DEBUG(MSG_WIN_API,
+	    Sdprintf("LoadLibraryExW() flags are supported\n"));
+    } else
+    { DEBUG(MSG_WIN_API,
+	    Sdprintf("LoadLibraryExW() flags are NOT supported\n"));
     }
     done = TRUE;
   }
@@ -690,7 +696,11 @@ PRED_IMPL("win_add_dll_directory", 2, win_add_dll_directory, 0)
       return PL_representation_error("file_name");
     if ( load_library_search_flags() )
     { if ( (cookie = (*f_AddDllDirectoryW)(dirw)) )
+      { DEBUG(MSG_WIN_API,
+	      Sdprintf("AddDllDirectory(%Ws) ok\n", dirw));
+
 	return PL_unify_int64(A2, (int64_t)(uintptr_t)cookie);
+      }
       return PL_error(NULL, 0, WinError(), ERR_SYSCALL, "AddDllDirectory()");
     } else
       return FALSE;
@@ -743,6 +753,8 @@ PL_dlopen(const char *file, int flags)	/* file is in UTF-8, POSIX path */
     return NULL;
   }
 
+  DEBUG(MSG_WIN_API, Sdprintf("dlopen(%Ws)\n", wfile));
+
   if ( is_windows_abs_path(wfile) )
     llflags |= load_library_search_flags();
 
