@@ -463,22 +463,21 @@ typedef struct
   int (*function)();			/* function to generate */
 } tprop;
 
-COMMON(int)	get_prop_def(term_t t, atom_t expected,
-			     const tprop *list, const tprop **def);
-
-COMMON(void)	initPrologThreads(void);
-COMMON(int)	pl_atom_table_in_use(AtomTable atom_table);
-COMMON(int)	pl_atom_bucket_in_use(Atom *atom_bucket);
-COMMON(Atom**)	pl_atom_buckets_in_use(void);
+COMMON(int)		get_prop_def(term_t t, atom_t expected,
+				     const tprop *list, const tprop **def);
+COMMON(void)		initPrologThreads(void);
+COMMON(int)		pl_atom_table_in_use(AtomTable atom_table);
+COMMON(int)		pl_atom_bucket_in_use(Atom *atom_bucket);
+COMMON(Atom**)		pl_atom_buckets_in_use(void);
 COMMON(Definition*)	predicates_in_use(void);
-COMMON(int)	pl_functor_table_in_use(FunctorTable functor_table);
-COMMON(int)	pl_kvs_in_use(KVS kvs);
-COMMON(gen_t)	pushPredicateAccess__LD(Definition def ARG_LD);
-COMMON(void)	popPredicateAccess__LD(Definition def ARG_LD);
-COMMON(size_t)	popNPredicateAccess__LD(size_t n ARG_LD);
-COMMON(void)	markAccessedPredicates(PL_local_data_t *ld);
-COMMON(int)     cgc_thread_stats(cgc_stats *stats ARG_LD);
-COMMON(int)	signalGCThread(int sig);
-COMMON(int)	isSignalledGCThread(int sig ARG_LD);
+COMMON(int)		pl_functor_table_in_use(FunctorTable functor_table);
+COMMON(int)		pl_kvs_in_use(KVS kvs);
+COMMON(definition_ref*) pushPredicateAccessObj(Definition def ARG_LD);
+COMMON(void)		popPredicateAccess__LD(Definition def ARG_LD);
+COMMON(size_t)		popNPredicateAccess__LD(size_t n ARG_LD);
+COMMON(void)		markAccessedPredicates(PL_local_data_t *ld);
+COMMON(int)		cgc_thread_stats(cgc_stats *stats ARG_LD);
+COMMON(int)		signalGCThread(int sig);
+COMMON(int)		isSignalledGCThread(int sig ARG_LD);
 
 #endif /*PL_THREAD_H_DEFINED*/
